@@ -193,7 +193,7 @@ class XQuerySet
 		}
 		if(i>=HDRS_NB)
 		{
-			if(verbose>1) i_error("FTS Xapian: Unknown header (lookup) '%s'",h2);
+			if(fts_xapian_settings.verbose>1) i_error("FTS Xapian: Unknown header (lookup) '%s'",h2);
 			i_free(h2); i_free(t2);
 			return;
 		}
@@ -327,7 +327,7 @@ class XQuerySet
 class XNGram
 {
 	private:
-		long partial,full,hardlimit;
+		long hardlimit;
 		const char * prefix;
 		bool onlyone;
 		icu::Transliterator *accentsConverter;
@@ -337,9 +337,8 @@ class XNGram
 		long size,maxlength;
 		long memory;
 
-	XNGram(long p, long f, const char * pre)
+	XNGram(const char * pre)
 	{
-		partial=p; full=f;
 		size=0;
 		memory=0;
 		maxlength=0;
@@ -414,7 +413,7 @@ class XNGram
 		}
 
 		long l = d->length();
-		if(l<partial) return;
+		if(l<fts_xapian_settings.partial) return;
 
 		if(accentsConverter == NULL)
 		{
@@ -434,30 +433,30 @@ class XNGram
 			return;
 		}
 
-		for(i=0;i<=l-partial;i++)
+		for(i=0;i<=l-fts_xapian_settings.partial;i++)
 		{
-			for(long j=partial;(j+i<=l)&&(j<=full);j++)
+			for(long j=fts_xapian_settings.partial;(j+i<=l)&&(j<=fts_xapian_settings.full);j++)
 			{
 				r = new icu::UnicodeString(*d,i,j);
 				add_stem(r);
 				delete(r);
 			}
 		}
-		if(l>full) add_stem(d);
+		if(l>fts_xapian_settings.full) add_stem(d);
 	}
 
 	void add_stem(icu::UnicodeString *d)
 	{
 		d->trim();
 		long l=d->length();
-		if(l<partial) return;
+		if(l<fts_xapian_settings.partial) return;
 
 		std::string s;
 		d->toUTF8String(s);
 		l  = s.length();
 		if(l>hardlimit)
 		{
-			if(verbose>0) i_warning("FTS Xapian: Term too long to be indexed (%s ...)",s.substr(0,100).c_str());
+			if(fts_xapian_settings.verbose>0) i_warning("FTS Xapian: Term too long to be indexed (%s ...)",s.substr(0,100).c_str());
 			return;
 		}
 
@@ -504,100 +503,60 @@ static long fts_backend_xapian_current_time()
 	return tp.tv_sec * 1000 + tp.tv_usec / 1000;
 }
 
-static long fts_backend_xapian_memory_used() // KB
+static long fts_backend_xapian_get_free_memory() // KB
 {
-	FILE* file = fopen("/proc/self/status", "r");
-	int i;
-	char line[129];
-	const char* p;
+	struct rlimit rl;
+	getrlimit(RLIMIT_AS,&rl);
 
-	if(file != NULL)
-	{
-		while (fgets(line, 128, file) != NULL)
-		{
-			if (strncmp(line, "VmSize:", 7) == 0)
-			{
-				i = strlen(line);
-				p = line+7;
-				while (*p <'0' || *p > '9') p++;
-				line[i-3] = '\0';
-				fclose(file);
-				return atol(p);
-			}
-		}
-		fclose(file);
-	}
-	return 0;
+	long limit = rl.rlim_cur / 1024.0;
+	if(fts_xapian_settings.verbose>1) i_warning("FTS Xapian: RLIM AS =%ld",limit);
+
+	getrlimit(RLIMIT_DATA,&rl);
+	long l2 = rl.rlim_cur / 1024.0;
+        if(fts_xapian_settings.verbose>1) i_warning("FTS Xapian: RLIM DATA =%ld",l2);
+
+	if((l2>0) && ((limit>l2) || (limit<1))) limit=l2;
+	
+#ifdef __FreeBSD__
+	uint32_t m;
+	size_t len = sizeof(m);
+	sysctlbyname("vm.stats.vm.v_free_count", &m, &len, NULL, 0);
+	if(fts_xapian_settings.verbose>1) i_warning("FTS Xapian: (BSD) Free pages %ld",long(m));
+	m = m * fts_xapian_settings.pagesize / 1024.0;
+	if((limit>0) && (m>limit)) m = limit;
+	if(fts_xapian_settings.verbose>1) i_warning("FTS Xapian: (BSD) Free memory %ld MB",long(m/1024.0));
+	return long(m);
+#else
+	long m = long(sysconf(_SC_AVPHYS_PAGES) * fts_xapian_settings.pagesize / 1024.0);
+	if((limit>0) && (m>limit)) m = limit;
+	if(fts_xapian_settings.verbose>1) i_warning("FTS Xapian: Free memory %ld MB",long(m/1024.0));
+	return m;
+#endif
 }
 
-static long fts_backend_xapian_memory_free() // KB
+static bool fts_backend_xapian_test_memory()
 {
-	FILE* file = fopen("/proc/meminfo", "r");
-	int i;
-	char line[129];
-	const char* p;
+	long fri = fts_backend_xapian_get_free_memory(); // KB
 
-	if(file != NULL)
-	{
-		while (fgets(line, 128, file) != NULL)
-		{
-			if (strncmp(line, "MemFree:", 8) == 0)
-			{
-				i = strlen(line);
-				p = line+8;
-				while (*p <'0' || *p > '9') p++;
-				line[i-3] = '\0';
-				fclose(file);
-				return atol(p);
-			}
-		}
-		fclose(file);
-	}
-	return 0;
-}
+	if(fts_xapian_settings.verbose>1) i_warning("FTS Xapian: Free memory %ld MB vs %ld MB minimum",long(fri/1024.0),fts_xapian_settings.lowmemory);
 
-static bool fts_backend_xapian_test_memory(struct xapian_fts_backend *backend, long add)
-{
-	rlim_t limit;
-
-	restrict_get_process_size(&limit);
-	long m = long((long)limit / 1024.0); // vsz_limit of dovecot.conf
-	long used = fts_backend_xapian_memory_used();
-	long fri = fts_backend_xapian_memory_free(); // Free RAM
-
-	backend->nb_pushes++;
-	long m2 = 2*used/backend->nb_pushes;
-	if(backend->max_push < m2) backend->max_push=m2;
-	m2=backend->max_push;
-
-	add = long(add/1024.0);
-
-	if(m<1)
-	{
-		if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB (%ld pushes), Free = %ld MB, Additional data %ld KB, Estimated required = %ld MB",long(used/1024), backend->nb_pushes, long(fri/1024), add, long(m2/1024));
-		return ((fri>XAPIAN_MIN_RAM*1024)&&(fri>m2));
-	}
-	else
-	{
-		if(verbose>0) i_info("FTS Xapian: Memory stats : Used = %ld MB (%ld%%) (%ld pushes), Limit = %ld MB, Free = %ld MB, Additional data %ld KB, Estimated required = %ld MB",long(used/1024),long(used*100.0/m),backend->nb_pushes,long(m/1024),long(fri/1024), add, long(m2/1024));
-		return ((fri>XAPIAN_MIN_RAM*1024)&&(m>(used+m2))&&(fri>m2));
-	}
+	return(fri > ( fts_xapian_settings.lowmemory * 1024 ) );
 }
 
 static bool fts_backend_xapian_open_readonly(struct xapian_fts_backend *backend, Xapian::Database ** dbr)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_open_readonly");
+	if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: fts_backend_xapian_open_readonly");
 
 	if((backend->db == NULL) || (strlen(backend->db)<1))
 	{
-		if(verbose>0) i_warning("FTS Xapian: Open DB Read Only : no DB name");
+		if(fts_xapian_settings.verbose>0) i_warning("FTS Xapian: Open DB Read Only : no DB name");
 		return false;
 	}
 
 	try
 	{
-		if(verbose>1) i_info("FTS Xapian: Opening DB (RO) %s",backend->db);
-		*dbr = new Xapian::Database(backend->db,Xapian::DB_OPEN);
+		if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: Opening DB (RO) %s",backend->db);
+		*dbr = new Xapian::Database(backend->db,Xapian::DB_OPEN | Xapian::DB_NO_SYNC);
 	}
 	catch(Xapian::Error e)
 	{
@@ -609,11 +568,11 @@ static bool fts_backend_xapian_open_readonly(struct xapian_fts_backend *backend,
 
 static bool fts_backend_xapian_check_access(struct xapian_fts_backend *backend)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_check_access");
+	if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: fts_backend_xapian_check_access");
 
 	if((backend->db == NULL) || (strlen(backend->db)<1))
 	{
-		if(verbose>0) i_warning("FTS Xapian: check_write : no DB name");
+		if(fts_xapian_settings.verbose>0) i_warning("FTS Xapian: check_write : no DB name");
 		return false;
 	}
 
@@ -621,21 +580,25 @@ static bool fts_backend_xapian_check_access(struct xapian_fts_backend *backend)
 
 	try
 	{
-		if(verbose>0) i_info("FTS Xapian: Opening DB (RW) %s",backend->db);
-		backend->dbw = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Opening DB (RW) %s",backend->db);
+		backend->dbw = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS | Xapian::DB_NO_SYNC);
 	}
 	catch(Xapian::Error e)
 	{
-		i_error("FTS Xapian: Can't open Xapian DB (%s) %s : %s - %s",backend->boxname,backend->db,e.get_type(),e.get_error_string());
+		i_error("FTS Xapian: Can't open Xapian DB (RW) (%s) %s : %s - %s",backend->boxname,backend->db,e.get_type(),e.get_error_string());
 		return false;
 	}
-	if(verbose>0) i_info("FTS Xapian: Opening DB (RW) %s : Done",backend->db);
+	if(fts_xapian_settings.verbose>0) 
+	{
+		long n = backend->dbw->get_doccount();
+		i_info("FTS Xapian: Opening DB (RW) %s (%ld docs stored): Done",backend->db,n);
+	}
 	return true;
 }
 
 static void fts_backend_xapian_oldbox(struct xapian_fts_backend *backend)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_oldbox");
+	if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: fts_backend_xapian_oldbox");
 
 	if(backend->old_guid != NULL)
 	{
@@ -649,7 +612,7 @@ static void fts_backend_xapian_oldbox(struct xapian_fts_backend *backend)
 		}
 		/* End Performance calculator*/
 
-		if(verbose>0) i_info("FTS Xapian: Done indexing '%s' (%s) (%ld msgs in %ld ms, rate: %.1f)",backend->old_boxname, backend->old_guid,backend->perf_nb,dt,r);
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Done indexing '%s' (%s) (%ld msgs in %ld ms, rate: %.1f)",backend->old_boxname, backend->old_guid,backend->perf_nb,dt,r);
 
 		i_free(backend->old_guid); backend->old_guid = NULL;
 		i_free(backend->old_boxname); backend->old_boxname = NULL;
@@ -660,12 +623,14 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
 {
 	bool err=false;
 
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_release (%s)",reason);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_release (%s)",reason);
 
 	if(commit_time<1) commit_time = fts_backend_xapian_current_time();
 
+	long n = 0;
 	if(backend->dbw !=NULL)
 	{
+		if(fts_xapian_settings.verbose>0) n = backend->dbw->get_doccount();
 		try
 		{
 			backend->dbw->commit();
@@ -684,10 +649,10 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
 
 	if(err)
 	{
-		if(verbose>0) i_info("FTS Xapian: Re-creating index database due to error");
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Re-creating index database due to error");
 		try
 		{
-			Xapian::WritableDatabase * db = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OVERWRITE | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS);
+			Xapian::WritableDatabase * db = new Xapian::WritableDatabase(backend->db,Xapian::DB_CREATE_OR_OVERWRITE | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS | Xapian::DB_NO_SYNC);
 			db->close();
 			delete(db);
 		}
@@ -697,18 +662,22 @@ static void fts_backend_xapian_release(struct xapian_fts_backend *backend, const
 		}
 	}
 
-	backend->nb_pushes=0;
-	backend->max_push=0;
-
-	if(verbose>0)
+	if(fts_xapian_settings.verbose>0)
 	{
-		i_info("FTS Xapian: Committed '%s' in %ld ms",reason,fts_backend_xapian_current_time() - commit_time);
+		if(n>0)
+		{
+			i_info("FTS Xapian: Committed '%s' in %ld ms (%ld docs in index)",reason,fts_backend_xapian_current_time() - commit_time,n);
+		}
+		else
+		{
+			i_info("FTS Xapian: Committed '%s' in %ld ms",reason,fts_backend_xapian_current_time() - commit_time);
+		}
 	}
 }
 
 XResultSet * fts_backend_xapian_query(Xapian::Database * dbx, XQuerySet * query, long limit=0)
 {
-	if(verbose>1) i_info("FTS Xapian: fts_backend_xapian_query");
+	if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: fts_backend_xapian_query");
 
 	XResultSet * set= new XResultSet();
 
@@ -747,13 +716,13 @@ XResultSet * fts_backend_xapian_query(Xapian::Database * dbx, XQuerySet * query,
 
 static void fts_backend_xapian_do_expunge(const char *fpath)
 {
-	Xapian::WritableDatabase * dbw;
+	Xapian::WritableDatabase * dbw = NULL;
 
 	long dt = fts_backend_xapian_current_time();
 
 	try
 	{
-		dbw = new Xapian::WritableDatabase(fpath,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS);
+		dbw = new Xapian::WritableDatabase(fpath,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS | Xapian::DB_NO_SYNC);
 	}
 	catch(Xapian::Error e)
 	{
@@ -761,58 +730,104 @@ static void fts_backend_xapian_do_expunge(const char *fpath)
 		return;
 	}
 
-	XResultSet * result;
+	XResultSet * result = NULL;
+	XQuerySet * xq = new XQuerySet();
 
 	try
 	{
-		XQuerySet * xq = new XQuerySet();
 		xq->add(hdrs_emails[XAPIAN_EXPUNGE_HEADER],"1");
 		result=fts_backend_xapian_query(dbw,xq,1);
-		delete(xq);
 	}
 	catch(Xapian::Error e)
 	{
-		i_error("FTS Xapian: Expunging '%s' : %s - %s",fpath,e.get_type(),e.get_error_string());
-		dbw->close();
-		delete(dbw);
-		return;
+		i_error("FTS Xapian: Expunging (1) '%s' : %s - %s",fpath,e.get_type(),e.get_error_string());
+		if(result != NULL) { delete(result); result = NULL; }
 	}
 
+	delete(xq);
+
+	if(result == NULL) { dbw->close(); delete(dbw); return; }
+
 	Xapian::docid docid;
-	char * s;
 
 	long j=result->size;
-	if(verbose>0) i_info("FTS Xapian: Expunging '%s' : %ld to do",fpath,j);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Expunging (2) '%s' : %ld to do",fpath,j);
 
 	while(j>0)
 	{
-		docid=result->data[j-1];
-		if(docid>0)
+		if(dbw != NULL) 
+		{
+			if(!fts_backend_xapian_test_memory())
+			{
+				if(fts_xapian_settings.verbose>0) i_warning("FTS Xapian: Expunging with low memory (%ld MB)-> Committing on disk",long(fts_backend_xapian_get_free_memory()/1024.0));
+				dbw->commit();
+				dbw->close();
+				delete(dbw);
+				dbw=NULL;
+			}
+		}
+
+		if(dbw == NULL)
 		{
 			try
-			{
-				if(verbose>0) i_info("FTS Xapian: Expunging UID=%d '%s'",docid,fpath);
-				dbw->delete_document(docid);
+        		{
+				dbw = new Xapian::WritableDatabase(fpath,Xapian::DB_CREATE_OR_OPEN | Xapian::DB_RETRY_LOCK | Xapian::DB_BACKEND_GLASS | Xapian::DB_NO_SYNC);
 			}
 			catch(Xapian::Error e)
 			{
-				i_error("FTS Xapian: Expunging UID=%d '%s' : %s - %s",docid,fpath,e.get_type(),e.get_error_string());
+				i_error("FTS Xapian: Can't reopen Xapian DB %s : %s - %s",fpath,e.get_type(),e.get_error_string());
+				delete(result);
+				return;
 			}
+			catch(const std::exception &e)
+                        {
+				i_error("FTS Xapian: Can't reopen Xapian DB for expunging (%s) : Low memory (%s)",fpath, e.what());
+				delete(result);
+				return;
+                        }
+		}
+
+		docid=result->data[j-1];
+		try
+		{
+			if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Expunging (3a) UID=%d (Free mem= %ld MB) '%s'",docid,long(fts_backend_xapian_get_free_memory()/1024.0),fpath);
+			if(docid>0) dbw->delete_document(docid);
+			if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Expunging (3b) UID=%d (Free mem= %ld MB) done",docid,long(fts_backend_xapian_get_free_memory()/1024.0));
+			j--;
+		}
+		catch(Xapian::Error e)
+		{
+			i_error("FTS Xapian: Expunging (4a) UID=%d '%s' : %s - %s",docid,fpath,e.get_type(),e.get_error_string());
+			dbw->commit();
+			dbw->close();
+			delete(dbw);
+			dbw=NULL;
+		}
+		catch(const std::exception &e2)
+		{
+			i_error("FTS Xapian: Expunging (4b) UID=%d '%s' : %s",docid,fpath,e2.what());
+			dbw->commit();
+			dbw->close();
+			delete(dbw);
+			dbw=NULL;
 		}
-		j--;
 	}
 	delete(result);
-	dbw->commit();
-	dbw->close();
-	delete(dbw);
 	
+	if(dbw != NULL)
+	{
+		dbw->commit();
+		dbw->close();
+		delete(dbw);
+	}
+
 	dt = fts_backend_xapian_current_time() - dt;
-	i_info("FTS Xapian: Expunging '%s' done in %.2f secs",fpath,dt/1000.0);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Expunging (5) '%s' done in %.2f secs",fpath,dt/1000.0);
 }
 
 static int fts_backend_xapian_unset_box(struct xapian_fts_backend *backend)
 {
-	if(verbose>0) i_info("FTS Xapian: Unset box '%s' (%s)",backend->boxname,backend->guid);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Unset box '%s' (%s)",backend->boxname,backend->guid);
 
 	long commit_time = fts_backend_xapian_current_time();
 
@@ -839,16 +854,16 @@ static int fts_backend_xapian_set_path(struct xapian_fts_backend *backend)
 	struct mail_namespace * ns = backend->backend.ns;
 	if(ns->alias_for != NULL)
 	{
-		if(verbose>0) i_info("FTS Xapian: Switching namespace");
+		if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Switching namespace");
 		ns = ns->alias_for;
 	}
 
 	const char * path = mailbox_list_get_root_forced(ns->list, MAILBOX_LIST_PATH_TYPE_INDEX);
 
 	if(backend->path != NULL) i_free(backend->path);
-	backend->path = i_strconcat(path, "/" XAPIAN_FILE_PREFIX, NULL);
+	backend->path = i_strconcat(path, "/" XAPIAN_FILE_PREFIX, static_cast<const char*>(NULL));
 
-	if(verbose>0) i_info("FTS Xapian: Index path = %s",backend->path);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Index path = %s",backend->path);
 
 	struct stat sb;
 	if(!( (stat(backend->path, &sb)==0) && S_ISDIR(sb.st_mode)))
@@ -867,14 +882,14 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 	if (box == NULL)
 	{
 		if(backend->guid != NULL) fts_backend_xapian_unset_box(backend);
-		if(verbose>1) i_info("FTS Xapian: Box is empty");
+		if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: Box is empty");
 		return 0;
 	}
 
 	const char * mb;
 	fts_mailbox_get_guid(box, &mb );
 
-	if(verbose>0) i_info("FTS Xapian: Set box '%s' (%s)",box->name,mb);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: Set box '%s' (%s)",box->name,mb);
 
 	if((mb == NULL) || (strlen(mb)<3))
 	{
@@ -884,7 +899,7 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 
 	if((backend->guid != NULL) && (strcmp(mb,backend->guid)==0))
 	{
-		if(verbose>1) i_info("FTS Xapian: Box is unchanged");
+		if(fts_xapian_settings.verbose>1) i_info("FTS Xapian: Box is unchanged");
 		return 0;
 	}
 
@@ -892,10 +907,7 @@ static int fts_backend_xapian_set_box(struct xapian_fts_backend *backend, struct
 
 	if(fts_backend_xapian_set_path(backend)<0) return -1;
 
-	struct timeval tp;
-	long current_time;
-
-	current_time = fts_backend_xapian_current_time();
+	long current_time = fts_backend_xapian_current_time();
 
 	backend->commit_updates = 0;
 	backend->commit_time = current_time;
@@ -1000,13 +1012,11 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 {
 	bool ok=true;
 
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_index_hdr");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_index_hdr");
 
 	Xapian::WritableDatabase * dbx = backend->dbw;
-	long p = backend->partial;
-	long f = backend->full;
 
-	if(data->length()<p) return true;
+	if(data->length()<fts_xapian_settings.partial) return true;
 
 	if(strlen(field)<1) return true;
 
@@ -1057,10 +1067,10 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 
 	if(!ok) return false;
 
-	XNGram * ngram = new XNGram(p,f,h);
+	XNGram * ngram = new XNGram(h);
 	ngram->add(data);
 
-	if(verbose>0)
+	if(fts_xapian_settings.verbose>0)
 	{
 		i_info("FTS Xapian: Ngram(%s) -> %ld items (total %ld KB)",h,ngram->size, ngram->memory/1024);
 	}
@@ -1087,9 +1097,9 @@ bool fts_backend_xapian_index_hdr(struct xapian_fts_backend *backend, uint uid,
 		{
 			dbx->replace_document(docid,*doc);
 		}
-		catch (std::bad_alloc& ba)
+		catch (const std::exception &e)
 		{
-			i_info("FTS Xapian: Memory too low (hdr) '%s'",ba.what());
+			i_warning("FTS Xapian: Memory too low (hdr) '%s'",e.what());
 			ok = false;
 		}
 	}
@@ -1103,13 +1113,11 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 {
 	bool ok = true;
 
-	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_index_text");
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: fts_backend_xapian_index_text");
 
 	Xapian::WritableDatabase * dbx = backend->dbw;
-	long p = backend->partial;
-	long f = backend->full;
 
-	if(data->length()<p) return true;
+	if(data->length()<fts_xapian_settings.partial) return true;
 
 	XQuerySet * xq = new XQuerySet();
 
@@ -1118,7 +1126,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 
 	XResultSet * result=fts_backend_xapian_query(dbx,xq,1);
 
-	Xapian::docid docid;
+	Xapian::docid docid = 0;
 	Xapian::Document * doc = NULL;
 
 	try
@@ -1173,7 +1181,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 	long n = doc2->termlist_count();
 	Xapian::TermIterator * ti = new Xapian::TermIterator(doc2->termlist_begin());
 
-	XNGram * ngram = new XNGram(p,f,h);
+	XNGram * ngram = new XNGram(h);
 	const char * c;
 	while(n>0)
 	{
@@ -1187,7 +1195,7 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 		n--;
 	}
 
-	if(verbose>0) i_info("FTS Xapian: NGRAM(%s,%s) -> %ld items, max length=%ld, (total %ld KB)",field,h,ngram->size,ngram->maxlength,ngram->memory/1024);
+	if(fts_xapian_settings.verbose>0) i_info("FTS Xapian: NGRAM(%s,%s) -> %ld items, max length=%ld, (total %ld KB)",field,h,ngram->size,ngram->maxlength,ngram->memory/1024);
 
 	char *t = (char*)i_malloc(sizeof(char)*(ngram->maxlength+6));
 	for(n=0;n<ngram->size;n++)
@@ -1215,9 +1223,9 @@ bool fts_backend_xapian_index_text(struct xapian_fts_backend *backend,uint uid,
 		{
 			dbx->replace_document(docid,*doc);
 		}
-		catch (std::bad_alloc& ba)
+		catch (const std::exception &e)
 		{
-			i_info("FTS Xapian: Memory too low (text) '%s'",ba.what());
+			i_warning("FTS Xapian: Memory too low (text) '%s'",e.what());
 			ok = false;
 		}
 	}
